<?php

/**
 * Yii Framework FriendlyUrl Extension
 * @package FriendlyURL
 * @license BSD
 */

/**
 * Abstract controller for inheritance of Friendly URL pages.
 * It contains initialization of {@link CFriendlyUrlController::$furl} (friendly
 * URL data), tweaking {@link CController::$pageTitle} and view path.
 * Also it contains rewrited {@link CController::$createUrl} method with
 * support Friendly URL extension.
 *
 * All controllers, that called with Friendly Url extension must be extends
 * from CFriendlyUrlController.
 * @link http://www.yiiframework.com/doc/api/CController CController
 * @package FriendlyURL
 * @author Oleg Blednov aka Blake-R
 * @version 2009-04-13
 */
abstract class CFriendlyUrlController extends CController
{
	/**
	 * Friendly URL data.
	 * @var stdObject
	 * @see CFriendlyUrlManager::$lastParse
	 */
	protected $furl;
	/**
	 * Array of page names depending on current action.
	 * @var array
	 */
	protected $actionTitles = array();

	/**
	 * In addition to CContoller constructor setting up a {@link $furl} and
	 * CController::$pageTitle.
	 * @param string $id Id of this controller.
	 * @param CWebModule $model The module that this controller belongs to.
	 * @uses $actionTitles
	 * @uses $furl
	 * @uses CFriendlyUrlHelper::magicString()
	 * @uses CFriendlyUrlManager::$lastParse
	 * @uses CWebApplication::$urlManager
	 * @link http://www.yiiframework.com/doc/api/CController#__construct-detail CController::__construct()
	 * @link http://www.yiiframework.com/doc/api/CWebModule CWebModule
	 * @link http://www.yiiframework.com/doc/api/CWebApplication#urlManager-detail CWebApplication::$urlManager
	 */
	public function __construct($id, CWebModule $model=NULL)
	{
		parent::__construct($id, $model);

		$this->furl = Yii::app()->urlManager->lastParse;

		if (is_array($this->actionTitles[$this->furl->action]))
		{
			if (isset($this->actionTitles[$this->furl->action][$this->furl->page->pageId]))
			{
				$this->pageTitle = $this->actionTitles[$this->furl->action][$this->furl->page->pageId];
			}
		}
		elseif (isset($this->actionTitles[$this->furl->action]))
		{
			$this->pageTitle = $this->actionTitles[$this->furl->action];
		}
		else
		{
			$this->pageTitle = CFriendlyUrlHelper::magicString($this->furl->page->pageTitle, $this->furl);
		}
	}

	/**
	 * Returns the directory containing view files for this controller.
	 * Finding it in that way:
	 *      - protected/views/ControllerID/PageCname'
	 *      - protected/views/ControllerID'
	 * @return string The directory containing the view files for this controller.
	 * @uses $furl
	 * @link http://www.yiiframework.com/doc/api/CController#getViewPath-detail CController::getViewPath()
	 */
	public function getViewPath()
	{
		$viewController = "protected/views/{$this->id}";
		$viewPage = "{$viewController}/{$this->furl->page->cname}";
		if (file_exists($viewPage)) return $viewPage;
		else return $viewController;
	}

	/**
	 * Creates a relative URL for the specified action defined in this controller.
	 * @param string|integer $url The URL. This should be in the friendly URL
	 * format (simple beautiful URL, desirable without action), or an ID of page:
	 * uniqueID is negative, pageID is positive.
	 * If it is blank, use current $furl data.
	 * @param array $params Additional GET parameters.
	 * @param integer $item ID of element to action on it.
	 * @param string $action Action name to call.
	 * @return string Created friendly URL.
	 * @uses $furl
	 * @uses CFriendlyUrlManager::createUrl()
	 * @link http://www.yiiframework.com/doc/api/CController#createUrl-detail CController::createUrl()
	 */
	public function createUrl($url=NULL, $params=array(), $item=NULL, $action='')
	{
		if (!$url) $url = implode('/', $this->furl->pathArray);
		else
		{
			if (is_string($url))
			{
				$_a = strrev($url);
				if (substr($_a, 0, 1)!='/' && substr($_a, 0, 5)!=strrev($this->furl->actionSuffix).'.' && !$action)
				{
					// Only action specified in url, as CUrlManager describe.
					$action = $url;
					$url = implode('/', $this->furl->pathArray).'/';
				}
			}
		}
		if (!$action)
		{
			if (!$this->furl->action) $action = $this->defaultAction;
			elseif ($this->furl->action!=$this->defaultAction) $action = $this->furl->action;
		}
		if ($action==$this->defaultAction && !$item) $action = '';

		return Yii::app()->urlManager->createUrl($url, $params, $item, $action);
	}

	/**
	 * Make $furl read-only variable.
	 * @return stdObject $furl.
	 * @uses $furl
	 */
	public function getFurl()
	{
		return $this->furl;
	}

	/**
	 * Default action index with 404 http exception for security reasons.
	 * @uses CHttpException::__construct()
	 * @see CInlineAction
	 * @link http://www.yiiframework.com/doc/api/CHttpException#__construct-detail CHttpException::__construct()
	 * @link http://www.yiiframework.com/doc/api/CInlineAction CInlineAction
	 */
	public function actionIndex()
	{
		throw new CHttpException(404);
	}
}


?>
