<?php

/**
 * Yii Framework FriendlyUrl Extension
 * @package FriendlyURL
 * @license BSD
 */

/**
 * URL manager in replacement of CUrlManager with support of beauty URLs.
 * It generate and parse URLs in that maner:
 *      <pre>Yii::app()->baseUrl./&lt;path&gt;/&lt;action&gt;&lt;itemSplitter&gt;&lt;titem&gt;.&lt;actionSuffix&gt;></pre>
 *
 * For use it change class name of urlManager in Yii configuration file
 * (usually 'protected/config/main.php') to the class 'CFriendlyUrlManager'.
 * Additionally, you cat specify next parameters:
 *      - $itemSplitter
 *      - $actionSuffix
 *      - $caseSensitive
 *      - $furlPageStorageClass
 *      - $furlPageStorageParams
 * @link http://www.yiiframework.com/doc/api/CUrlManager CUrlManager
 * @package FriendlyURL
 * @author Oleg Blednov aka Blake-R
 * @version 2009-04-13
 */
class CFriendlyUrlManager extends CUrlManager
{
	/**
	 * Action and item splitter character.
	 * Default - dot.
	 * @var string
	 */
	public $itemSplitter = '.';
	/**
	 * Action suffix.
	 * Default - 'html'
	 * @var string
	 */
	public $actionSuffix = 'html';
	/**
	 * Whether url may be case insensitive or not.
	 * Default - false.
	 * @var boolean
	 */
	public $caseSensitive = false;

	/**
	 * Class name for pages storage.
	 * Now realized only two classes:
	 *      - CFriendlyUrlPagePhpStorage for reading pages from PHP-file and
	 *      - CFriendlyUrlPageDbStorage for reading pages from DB.
	 * Default - CFriendlyUrlPgePhpStorage.
	 * @var string
	 * @see CFriendlyUrlPagePhpStorage
	 * @see CFriendlyUrlPageDbStorage
	 */
	public $furlPageStorageClass = 'CFriendlyUrlPagePhpStorage';
	/**
	 * Additional parameters for initialization of storage class.
	 * Default - empty array.
	 * @var array
	 */
	public $furlPageStorageParams = array();

	/**
	 * Contains last parsed URL data:
	 *      - string $url            - bautifurized current URL,
	 *      - array $pathArray       - path elements (web directories),
	 *      - array $parentsArray    - all parents pages,
	 *      - CFriendlyUrlPage $page - current page,
     *      - string $action         - requested action,
	 *      - integer $item          - requested item for action.
	 * @var stdObject
	 * @see CFriendlyUrlPage
	 */
	protected $lastParse;
	/**
	 * Representative of page control class.
	 * @var CFriendlyUrlPageControl
	 * @see CFriendlyUrlPageControl
	 */
	protected $pageControl;

	/**
	 * Initialize a class: calling CUrlManager::init() and create page control.
	 * @uses $furlPageStorageClass
	 * @uses $furlPageStorageParams
	 * @uses $pageControl
	 * @uses CFriendlyUrlPageControl::__construct()
	 * @uses IFriendlyUrlPageStorage::__construct()
	 * @link http://www.yiiframework.com/doc/api/CUrlManager#init-detail CUrlManager::init()
	 */
	public function init()
	{
		parent::init();
		$this->pageControl = new CFriendlyUrlPageControl(
			new $this->furlPageStorageClass($this->furlPageStorageParams)
			);
	}

	/**
	 * Parse URL into a page, action and item, if that presents.
	 * @param CHttpRequest $request URL for parse.
	 * @return string Route to controller.
	 * @uses $lastParse
	 * @uses parseAction()
	 * @uses splitUrl()
	 * @uses CFriendlyUrlPageControl::findChild()
	 * @uses CHttpException::__construct()
	 * @uses CHttpRequest::getPathInfo()
	 * @uses CWebApplication::$baseUrl
	 * @link http://www.yiiframework.com/doc/api/CHttpRequest#getPathInfo-detail CHttpRequest::getPathInfo()
	 * @link http://www.yiiframework.com/doc/api/CWebApplication#baseUrl-detail CWebApplication::$baseUrl
	 * @link http://www.yiiframework.com/doc/api/CHttpException#__construct-detail CHttpException::__construct()
	 */
	public function parseUrl(CHttpRequest $request)
	{
		$url = substr($request->getPathInfo(), strlen(Yii::app()->baseUrl));

		// Get PathArray
		$this->lastParse->pathArray = $this->splitUrl($url);
		// Get Action
		// Get Item
		list($this->lastParse->action, $this->lastParse->item) = $this->parseAction($this->lastParse->pathArray);

		// Get URL
		$this->lastParse->url = '/'.implode('/', $this->lastParse->pathArray).'/';
		if (isset($this->lastParse->action))
		{
			$this->lastParse->url .= $this->lastParse->action;
			if (isset($this->lastParse->item))
			{
				$this->lastParse->url .= '.'.$this->lastParse->item;
			}
			$this->lastParse->url .= '.'.$this->actionSuffix;
		}

		// Get Page
		// Get ParentsArray
		$this->lastParse->page = NULL;
		$this->lastParse->parentsArray = array();
		foreach ($this->lastParse->pathArray as $cname)
		{
			$p = CFriendlyUrlPageControl::findChild($cname.'/'.$this->lastParse->action, $this->lastParse->page);
			if (!$p) $p = CFriendlyUrlPageControl::findChild($cname, $this->lastParse->page);
			$this->lastParse->page = $p;
			if (!$this->lastParse->page) break;
			array_push($this->lastParse->parentsArray, $this->lastParse->page);
		}
		if ($this->lastParse->page!==NULL) array_pop($this->lastParse->parentsArray);


		list($controller, $action) = explode('/', $this->lastParse->page->controller);

		if (!$action) $action = $this->lastParse->action;
		else $this->lastParse->action = $action;

		$route = $controller.($action? '/'.$action: '');

		if (!$route) throw new CHttpException(404);
		else return $route;
	}

	/**
	 * Create an friendly URL.
	 * @param string|integer $route URL or ID: uniqueID is negative, pageID is positive.
	 * @param array $params Additional query params.
	 * @param integer $item Item ID for action.
	 * @param string $action Action name.
	 * @return string Friendly URL.
	 * @uses $actionSuffix
	 * @uses $itemSplitter
	 * @uses splitUrl()
	 * @uses parseAction()
	 * @uses CWebApplication::$baseUrl
	 * @uses CWebApplication::$defaultcontroller
	 * @uses CFriendlyUrlPageControl::findByUniqueId()
	 * @uses CFriendlyUrlPageControl::findByPageId()
	 * @link http://www.yiiframework.com/doc/api/CWebApplication#defaultController-detail CWebApplication::$defaultController
	 * @link http://www.yiiframework.com/doc/api/CWebApplication#baseUrl-detail CWebApplication::$baseUrl
	 */
	public function createUrl($route, array $params=array(), $item=NULL, $action=NULL)
	{
		if (($id = intval($route))!=0)
		{
			if ($id<0) $page = Yii::app()->urlManager->pageControl->findByUniqueId(abs($id));
			else $page = Yii::app()->urlManager->pageControl->findByPageId($id);

			if ($page)
			{
				$pathArray = array();
				do
				{
					list($cname) = explode('/', $page->cname);
					if ($cname) array_push($pathArray, $cname);
				}
				while($page = $page->parent);
				$url = implode('/', array_reverse($pathArray));
			}
			else $url = $route;
		}
		else
		{
			$pathArray = $this->splitUrl($route);
			list($_action, $_item) = $this->parseAction($pathArray);
			$url = implode('/', $pathArray);
			if ($_action) $action = $_action;
			if ($_item) $item = $_item;
		}

		if (substr($url, strlen($url)-1, 1)!='/') $url .= '/';
		if ($action) $url .= $action.($item>0? $this->itemSplitter.$item: '').'.'.$this->actionSuffix;

		if ($url==Yii::app()->defaultController/* && $item==NULL && !$action*/) $url = '';

		if (!is_array($params))
		{
			if (is_string($params)) $params = explode('&', $params);
			else $params = array();
		}

		if ($params)
		{
			$query = array();
			foreach ($params as $key=>$value)
				array_push($query, "{$key}=".urlencode($value));
			$url .= '?'.implode('&', $query);
		}

		return Yii::app()->baseUrl.'/'.$url;
	}

	/**
	 * Split URL into array
	 * @param string $url URL.
	 * @return array Array of URL path elements.
	 * @uses CWebApplication::$defaultController
	 * @link http://www.yiiframework.com/doc/api/CWebApplication#defaultController-detail CWebApplication::$defaultController
	 */
	protected function splitUrl($url=NULL)
	{
		if (!$url) $url = Yii::app()->defaultController;
		return preg_split('/\//', $url, 0, PREG_SPLIT_NO_EMPTY);
	}

	/**
	 * Find action and item, if presents.
	 * @param array &$pathArray Array of path elements.
	 * @return array Action name and item ID.
	 * @uses $actionSuffix
	 * @uses $itemSplitter
	 * @uses splitUrl()
	 */
	protected function parseAction(array &$pathArray)
	{
		$action = NULL;
		$item = NULL;

		$actionRegEx = '/([a-zA-Z]+)('.preg_quote($this->itemSplitter).'(\d+))?\.'.
			preg_quote($this->actionSuffix).'$/'.($this->caseSensitive? '': 'i');

		if (preg_match($actionRegEx, $pathArray[count($pathArray)-1], $matches))
		{
			array_pop($pathArray);
			$action = $matches[1];
			if (isset($matches[3])) $item = intval($matches[3]);

			if (empty($pathArray)) $pathArray = $this->splitUrl();
		}

		return array($action, $item);
	}

	/**
	 * Make $lastParse read-only variable.
	 * @return stdObject
	 * @uses $lastParse
	 */
	protected function getLastParse() { return $this->lastParse; }
	/**
	 * Make $pageControl read-only variable.
	 * @return CFriendlyUrlPage
	 * @uses $pageControl
	 */
	protected function getPageControl() { return $this->pageControl; }
}

?>
