<?php

/**
 * Yii Framework FriendlyUrl Extension
 * @package FriendlyURL
 * @license BSD
 */

/**
 * Page control class for finding children and getting parent pages.
 * @link http://www.yiiframework.com/doc/api/CComponent CComponent
 * @package FriendlyURL
 * @author Oleg Blednov aka Blake-R
 * @version 2009-04-13
 */
class CFriendlyUrlPageControl extends CComponent
{
	/**
	 * Array of all pages.
	 * @var array
	 * @see CFriendlyUrlPage
	 */
	static protected $data = array();


	/**
	 * Constructor, fill pages data array.
	 * @param IFriendlyUrlPageStorage $storage Storage for getting data.
	 * @uses $data
	 * @uses IFriendlyUrlPageStorage::getData()
	 */
	public function __construct(IFriendlyUrlPageStorage $storage)
	{
		self::$data = $storage->getData();
	}


	/**
	 * Find page by unique ID.
	 * @param integer $uniqueId Unique ID to find.
	 * @param CFriendlyUrlPage $parent Parent page for finding or NULL for root.
	 * @return CFriendlyUrlPage Founded page or NULL.
	 * @uses $data
	 * @uses findByUniqueId()
	 * @uses CFriendlyUrlPage::$children
	 * @uses CFriendlyUrlPage::$uniqueId
	 */
	static public function findByUniqueId($uniqueId, CFriendlyUrlPage $parent=NULL)
	{
		if ($parent) $root = $parent->children;
		else $root = self::$data;

		foreach ($root as $page)
		{
			if ($page->uniqueId==$uniqueId) return $page;
			if ($page->children)
			{
				$ret = self::findByUniqueId($uniqueId, $page);
				if ($ret) return $ret;
			}
		}

		return NULL;
	}

	/**
	 * Find page by page ID.
	 * @param integer $pageId Page ID to find.
	 * @param CFriendlyUrlPage $parent Parent page for finding or NULL for root.
	 * @return CFriendlyUrlPage Founded page or NULL.
	 * @uses $data
	 * @uses findByPageId()
	 * @uses CFriendlyUrlPage::$children
	 * @uses CFriendlyUrlPage::$pageId

	 */
	static public function findByPageId($pageId, CFriendlyUrlPage $parent=NULL)
	{
		if ($parent) $root = $parent->children;
		else $root = self::$data;

		foreach ($root as $page)
		{
			if ($page->pageId==$pageId) return $page;
			if ($page->children)
			{
				$ret = self::findByPageId($pageId, $page);
				if ($ret) return $ret;
			}
		}

		return NULL;
	}

	/**
	 * Find child page by parently-unique name.
	 * @param string $cname Parently-unique name of page to find.
	 * @param CFriendlyUrlPage $parent Parent page for finding or NULL for root.
	 * @return CFriendlyUrlPage Founded page or NULL.
	 * @uses $data
	 * @uses CFriendlyUrlPage::$children
	 * @uses CFriendlyUrlPage::$cname
	 */
	static public function findChild($cname, CFriendlyUrlPage $parent=NULL)
	{
		if ($parent) $root = $parent->children;
		else $root = self::$data;

		foreach ($root as $page)
		{
			if ($cname==$page->cname) return $page;
		}

		return NULL;
	}

	/**
	 * Get children pages.
	 * @param CFriendlyUrlPage $parent Parent page for finding or NULL for root.
	 * @return array Children pages.
	 * @uses $data
	 * @uses CFriendlyUrlPage::$children
	 */
	static public function getChildren(CFriendlyUrlPage $parent=NULL)
	{
		if ($parent) $root = $parent->children;
		else $root = self::$data;

		return $root;
	}
}

?>
