<?php

/**
 * Yii Framework FriendlyUrl Extension
 * @package FriendlyURL
 * @license BSD
 */

/**
 * Widget for build menu based on friendly URL pages data.
 * Example of usage:
 *      <? $this->widget('FriendlyUrlMenu', array(
 *          'levels'=>1, 'parentMatch'=>true)) ?>
 * @link http://www.yiiframework.com/doc/api/CWidget CWidget
 * @package FriendlyURL
 * @author Oleg Blednov aka Blake-R
 * @version 2009-04-13
 */
class FriendlyUrlMenu extends CWidget
{
	/**
	 * Default template for output elements. Contains templates for:
	 *      - with outerBegin menu or submenu output starts,
	 *      - with innerBegin element output starts,
	 *      - link output a link for menu element,
	 *      - with innerEnd element output ends,
	 *      - with outerEnd menu or submenu output ends.
	 * You can use next variables in templates:
	 *      - $class - contains string 'class="<classname>"', where <classname>
	 *                 is one of {@link $style};
	 *      - $link - URL to menu element;
	 *      - $pageTitle - page title;
	 *      - $menuTitle - page menu title;
	 * Allowing using in templates:
	 *      - innerBegin: $class, $pageTitle;
	 *      - link: $class, $link, $pageTitle, $menuTitle.
	 * @var array
	 * @see $tpl
	 */
	static private $defaultTpl = array(
		'outerBegin'=>"<ul>\n",
		'outerEnd'=>"</ul>\n",
		'innerBegin'=>"\t<li>",
		'innerEnd'=>"</li>\n",
		'link'=>"<a \$class href='\$link' title='\$pageTitle'>\$menuTitle</a>",
		);

	/**
	 * Parent page or PageID, children of whom your want to output in menu.
	 * @var CFriendlyUrlPage|integer
	 */
	public $parent = NULL;
	/**
	 * Menu depth to output.
	 * @var integer
	 */
	public $levels = 0;
	/**
	 * User templates for output.
	 * @var array
	 * @see FriendlyUrlMenu::$defaultTpl
	 */
	public $tpl = array();
	/**
	 * Two string elements - inactive and active - sets a classes to use when
	 * the menu element is current page and in all other cases.
	 * @var array
	 */
	public $style = array(
		'inactive'=>'',
		'active'=>'active'
		);
	/**
	 * Whether mean parents, what the parent element of current page is active,
	 * or not.
	 * @var boolean
	 */
	public $parentMatch = false;

	/**
	 * Current URL.
	 * @var string
	 */
	protected $currentUrl;

	/**
	 * Initialization widget. Set $tpl, compute parent page and get $currentUrl.
	 * @uses $tpl
	 * @uses $parent
	 * @uses buildSubMenu()
	 * @uses CFriendlyUrlController::$furl
	 * @uses CFriendlyUrlManager::$pageControl
	 * @uses CWebApplication::$urlManager
	 * @uses CWidget::$controller
	 * @uses FriendlyUrlMenu::$defaultTpl
	 * @uses CFriendlyUrlManager::createUrl()
	 * @uses CFriendlyUrlPageControl::findByPageId()
	 * @link http://www.yiiframework.com/doc/api/CWebApplication#urlManager-detail CWebApplication::$urlManager
	 * @link http://www.yiiframework.com/doc/api/CWidget#controller-detail CWidget::controller()
	 * @link http://www.yiiframework.com/doc/api/CWidget#run-detail CWidget::run()
	 */
	public function run()
	{
		$this->tpl = array_merge(self::$defaultTpl, $this->tpl);

		$parent = $this->parent;
		if (is_integer($parent))
		{
			$parent = Yii::app()->urlManager->pageControl->findByPageId($this->parent);
			if (!$parent)
			{
				return;
			}
		}

		$this->currentUrl = Yii::app()->urlManager->createUrl($this->controller->furl->url,
			array(), NULL, $this->controller->furl->action);

		$this->buildSubMenu($parent, 0);
	}

	/**
	 * Build menu.
	 * @param CFriendlyUrlPage $parent Parent page.
	 * @param integer $level Current nesting level.
	 * @uses $currentUrl
	 * @uses $levels
	 * @uses $parentMatch
	 * @uses $style
	 * @uses $tpl
	 * @uses buildSubMenu()
	 * @uses CFriendlyUrlPage::$cname
	 * @uses CFriendlyUrlPage::$menuTitle
	 * @uses CFriendlyUrlPage::$pageTitle
	 * @uses CFriendlyUrlPage::$uniqueId
	 * @uses CFriendlyUrlManager::$pageControl
	 * @uses CWebApplication::$urlManager
	 * @uses CFriendlyUrlHelper::magicString()
	 * @uses CFriendlyUrlPageControl::getChildren()
	 * @link http://www.yiiframework.com/doc/api/CWebApplication#urlManager-detail CWebApplication::$urlManager
	 */
	protected function buildSubMenu($parent, $level=0)
	{
		$pages = Yii::app()->urlManager->pageControl->getChildren($parent);
		if (!$pages) return;

		isset($this->tpl['outerBegin']) && print $this->tpl['outerBegin'];

		foreach ($pages as $page)
		{
			$pageTitle = CFriendlyUrlHelper::magicString($page->pageTitle);
			$menuTitle = CFriendlyUrlHelper::magicString($page->menuTitle);
			if (!empty($menuTitle))
			{
				list($cname, $action) = explode('/', $page->cname);
				$url = Yii::app()->urlManager->createUrl(-$page->uniqueId, array(), NULL, $action);

				if ($url==$this->currentUrl ||
					($this->parentMatch && strlen($url)>1 &&
						$url==substr($this->currentUrl, 0, strlen($url))))
				{
					$style = $this->style['active'];
				}
				else
				{
					$style = $this->style['inactive'];
				}
				$class = $style? "class='$style'": '';

				isset($this->tpl['innerBegin']) && print str_replace(
					array('$class', '$pageTitle'), array($class, $pageTitle),
					$this->tpl['innerBegin']);

				echo str_replace(array('$class', '$link', '$pageTitle','$menuTitle'),
						array($class, $url, $pageTitle, $menuTitle), $this->tpl['link']);

				if ($level<$this->levels || $this->levels==0)
					$this->buildSubMenu($page, $level+1);

				isset($this->tpl['innerEnd']) && print $this->tpl['innerEnd'];
			}
		}

		isset($this->tpl['outerEnd']) && print $this->tpl['outerEnd'];
	}
}

?>
